/**
 * Ein Element eines Binaerbaums.
 * Die Klasse ist abstrakt, da ein Baumelement
 * entweder ein Knoten (mit Daten) oder ein Abschluss
 * des Baums sein kann.
 * 
 * @author Bernd Gramlich 
 * @version 2012-12-20
 */
public abstract class Baumelement
{
    /*
     * Die Klasse selbst hat keinen Konstruktor, sondern nur
     * ihre Unterklassen.
     */

    /**
     * Nenne den linken Nachfolger des Baumelements.
     */
    public abstract Baumelement gibLinkenNachfolger ();

    /**
     * Nenne den rechten Nachfolger des Baumelements.
     */
    public abstract Baumelement gibRechtenNachfolger ();

    /**
     * Nenne den Inhalt des Baumelements.
     */
    public abstract Eintrag gibInhalt ();

    /**
     * Fge einen neuen Eintrag aufsteigend sortiert in den Baum ein.
     */
    public abstract Baumelement fuegePassendEin (Eintrag neuerEintrag, String baumposition);

    /**
     * Drucke den Inhalt des Binrbaums ab mir selbst in
     * aufsteigender Reihenfolge auf die Konsole.
     */
    public abstract void druckeInOrder ();

    /**
     * Drucke den Inhalt des Binrbaums ab mir selbst in
     * Prordnung auf die Konsole.
     */
    public abstract void druckePreOrder ();

    /**
     * Drucke den Inhalt des Binrbaums ab mir selbst in
     * Postordnung auf die Konsole.
     */
    public abstract void druckePostOrder ();

    /**
     * Zhle, wie viele Knoten ab mir selbst die "eigenschaft" haben.
     */
    public abstract int zaehle (Eigenschaft eigenschaft);

    /**
     * Fhre fr alle Knoten ab mir selbst, die die Eigenschaft
     * "eigenschaft" haben, den Befehl "befehl" mit den Argumenten
     * "zeichenkette" aus.
     */
    public abstract void fuehreAusFuer (Eigenschaft eigenschaft, Befehl befehl, String[] zeichenkette);

    /**
     * Suche einen Eintrag ab mir im Baum (in order), der zum Suchwort passt.
     */
    public abstract Eintrag suche (String suchwort); 

    /**
     * Lsche den ersten Knoten ab mir im Baum, der zum Suchwort passt.
     */
    public abstract Baumelement loesche (String suchwort, String baumposition);

    /**
     * Nenne den grten Eintrag ab mir im Baum.
     */
    public abstract Eintrag gibMaximum ();

    /**
     * Lsche den grten Knoten ab mir im Baum.
     */
    public abstract Baumelement loescheMaximum ();

    /**
     * Zeichne den Baum ab mir neu.
     */
    public abstract void zeichneNeu (String baumposition);
}
