/**
 * Ein allgemeiner Binaerbaum.
 * 
 * @author Bernd Gramlich 
 * @version 2012-12-20
 */
public class Binaerbaum
{
    private Baumelement wurzel;

    /**
     * Konstruktor.
     * Erzeuge einen leeren Binrbaum. Das ist also ein
     * Binrbaum, der nur aus einem Abschluss besteht.
     */
    public Binaerbaum ()
    {
        wurzel = new Abschluss ();
    }
    
    /**
     * Fge einen neuen Eintrag aufsteigend sortiert in den
     * Binrbaum ein.
     */
    public void fuegePassendEin (Eintrag neuerEintrag)
    {
        wurzel = wurzel.fuegePassendEin (neuerEintrag, "");
    }
    
    /**
     * Drucke den Inhalt des Binrbaums in aufsteigender
     * Reihenfolge auf die Konsole.
     */
    public void druckeInOrder ()
    {
        wurzel.druckeInOrder ();
        
        System.out.println ();
    }
    
    /**
     * Drucke den Inhalt des Binrbaums in Prordnung auf
     * die Konsole.
     */
    public void druckePreOrder ()
    {
        wurzel.druckePreOrder ();
        
        System.out.println ();
    }
    
    /**
     * Drucke den Inhalt des Binrbaums in Postordnung auf
     * die Konsole.
     */
    public void druckePostOrder ()
    {
        wurzel.druckePostOrder ();
        
        System.out.println ();
    }
    
    /**
     * Zhle, wie viele Knoten die "eigenschaft" haben.
     * 
     * Beispiel:
     * zaehle (new IstWeiblich ())
     * zhlt die Anzahl der Frauen im Binrbaum.
     */
    public int zaehle (Eigenschaft eigenschaft)
    {
        return wurzel.zaehle (eigenschaft);
    }
    
    /**
     * Fhre fr alle Knoten, die die Eigenschaft "eigenschaft"
     * haben, den Befehl "befehl" mit den Argumenten "zeichenkette"
     * aus.
     * 
     * Beispiele:
     * fuehreAusFuer (new IstWeiblich (), new Schwebe (), {});
     * lsst alle Frauen im Binrbaum nach oben schweben.
     * fuehreAusFuer (new IstWeiblich (), new FaerbeHaare (), {"rot"});
     * frbt die Haare aller Frauen im Binrbaum rot.
     * fuehreAusFuer (new IstPerson (), new SetzeKleiderfarben (),
     *                                    {"rot", "gelb", "grn"});
     * kleidet alle Personen im Binrbaum in Ampelfarben.
     */
    public void fuehreAusFuer (Eigenschaft eigenschaft, Befehl befehl, String[] zeichenkette)
    {
        wurzel.fuehreAusFuer (eigenschaft, befehl, zeichenkette);
    }
    
    /**
     * Suche einen Eintrag im Baum (in order), der zum Suchwort passt.
     */
    public Eintrag suche (String suchwort)
    {
        return wurzel.suche (suchwort);
    } 
    
    /**
     * Lsche den ersten Knoten im Baum, der zum Suchwort passt.
     */
    public void loesche (String suchwort)
    {
        wurzel = wurzel.loesche (suchwort, "");
    } 
}