/**
 * Ein Knoten eines allgemeinen Binrbaums.
 * 
 * @author Bernd Gramlich 
 * @version 2018-01-10
 */
public class Knoten extends Baumelement
{
    /*
     * Inhalt des Knotens ist ein Eintrag.
     */
    private Eintrag inhalt;

    /*
     * Nachfolger des Knotens sind zwei Baumelemente,
     * also entweder wieder Knoten oder Abschlsse
     * des Baums.
     */
    private Baumelement linkerNachfolger;
    private Baumelement rechterNachfolger;

    /**
     * Konstruktor.
     * Erzeuge einen neuen Knoten mit Inhalt "meinInhalt" und
     * Nachfolgern "meinLinkerNachfolger" und "meinRechterNachfolger"
     * an der "baumposition".
     */
    public Knoten (Baumelement meinLinkerNachfolger, Baumelement meinRechterNachfolger,
    Eintrag meinInhalt, String baumposition)
    {
        linkerNachfolger  = meinLinkerNachfolger;
        rechterNachfolger = meinRechterNachfolger;
        inhalt            = meinInhalt;

        inhalt.zeichneAnBaumposition (baumposition);
    }

    /**
     * Nenne den linken Nachfolger des Knotens.
     */
    public Baumelement gibLinkenNachfolger ()
    {
        return linkerNachfolger;
    }

    /**
     * Nenne den rechten Nachfolger des Knotens.
     */
    public Baumelement gibRechtenNachfolger ()
    {
        return rechterNachfolger;
    }

    /**
     * Nenne den Inhalt des Knotens.
     */
    public Eintrag gibInhalt ()
    {
        return inhalt;
    }

    /**
     * Verschaffe dem Knoten einen neuen linken Nachfolger.
     */
    public void setzeLinkenNachfolger (Baumelement neuerLinkerNachfolger)
    {
        linkerNachfolger = neuerLinkerNachfolger;
    }

    /**
     * Verschaffe dem Knoten einen neuen rechten Nachfolger.
     */
    public void setzeRechtenNachfolger (Baumelement neuerRechterNachfolger)
    {
        rechterNachfolger = neuerRechterNachfolger;
    }

    /**
     * Fge einen neuen Eintrag aufsteigend sortiert in den Baum ein.
     */
    public Baumelement fuegePassendEin (Eintrag neuerEintrag, String baumposition)
    {
        if (inhalt.istKleinerAls (neuerEintrag))
        {
            /*
             * Wenn mein Inahlt kleiner ist als der neue Eintrag, dann muss
             * der neue Eintrag zu meiner Rechten eingefgt werden. Diese
             * Aufgabe berlasse ich meinem rechten Nachfolger.
             */
            rechterNachfolger = rechterNachfolger.fuegePassendEin (neuerEintrag, baumposition + "r");

            return this;
        }
        else
        {
            /*
             * Wenn der neue Eintrag hchstens so gro ist wie mein
             * Inhalt, dann muss der neue Eintrag zu meiner Linken
             * eingefgt werden. Diese Aufgabe berlasse ich meinem
             * linken Nachfolger.
             */
            linkerNachfolger = linkerNachfolger.fuegePassendEin (neuerEintrag, baumposition + "l");

            return this;
        }
    }

    /**
     * Drucke den Inhalt des Binrbaums ab mir selbst in
     * aufsteigender Reihenfolge auf die Konsole.
     */
    public void druckeInOrder ()
    {
        linkerNachfolger.druckeInOrder ();
        inhalt.druckeAufKonsole ();
        rechterNachfolger.druckeInOrder ();
    }

    /**
     * Drucke den Inhalt des Binrbaums ab mir selbst in
     * Prordnung auf die Konsole.
     */
    public void druckePreOrder ()
    {
        inhalt.druckeAufKonsole ();
        linkerNachfolger.druckePreOrder ();
        rechterNachfolger.druckePreOrder ();
    }

    /**
     * Drucke den Inhalt des Binrbaums ab mir selbst in
     * Postordnung auf die Konsole.
     */
    public void druckePostOrder ()
    {
        linkerNachfolger.druckePostOrder ();
        rechterNachfolger.druckePostOrder ();
        inhalt.druckeAufKonsole ();
    }

    /**
     * Zhle, wie viele Knoten ab mir selbst die "eigenschaft" haben.
     */
    public int zaehle (Eigenschaft eigenschaft)
    {
        if (eigenschaft.hatEigenschaft (inhalt))
        {
            return 1 + linkerNachfolger.zaehle (eigenschaft)
            + rechterNachfolger.zaehle (eigenschaft);
        }
        else
        {
            return linkerNachfolger.zaehle (eigenschaft)
            + rechterNachfolger.zaehle (eigenschaft);
        }
    }

    /**
     * Fhre fr alle Knoten ab mir selbst, die die Eigenschaft
     * "eigenschaft" haben, den Befehl "befehl" mit den Argumenten
     * "zeichenkette" aus.
     */
    public void fuehreAusFuer (Eigenschaft eigenschaft, Befehl befehl, String[] zeichenkette)
    {
        if (eigenschaft.hatEigenschaft (inhalt))
        {
            befehl.fuehreAus (inhalt, zeichenkette);
        }

        linkerNachfolger.fuehreAusFuer (eigenschaft, befehl, zeichenkette);
        rechterNachfolger.fuehreAusFuer (eigenschaft, befehl, zeichenkette);
    }

    /**
     * Suche einen Eintrag ab mir im Baum (in order), der zum Suchwort passt.
     */
    public Eintrag suche (String suchwort)
    {
        if (inhalt.vergleicheMit (suchwort) > 0)
        {
            return linkerNachfolger.suche (suchwort);
        }
        else if (inhalt.vergleicheMit (suchwort) == 0)
        {
            return inhalt;
        }
        else
        {
            return rechterNachfolger.suche (suchwort);
        }
    }

    /**
     * Lsche den ersten Knoten ab mir im Baum, der zum Suchwort passt.
     */
    public Baumelement loesche (String suchwort, String baumposition)
    {
        if (inhalt.vergleicheMit (suchwort) > 0)
        {
           linkerNachfolger = linkerNachfolger.loesche (suchwort, baumposition + "l");
           
           return this;
        }
        else if (inhalt.vergleicheMit (suchwort) == 0)
        {
            inhalt.macheUnsichtbar ();

            if (linkerNachfolger instanceof Abschluss)
            {
                rechterNachfolger.zeichneNeu (baumposition);

                return rechterNachfolger;
            }
            else if (rechterNachfolger instanceof Abschluss)
            {
                linkerNachfolger.zeichneNeu (baumposition);

                return linkerNachfolger;
            }
            else
            {
                inhalt = linkerNachfolger.gibMaximum ();
                linkerNachfolger = linkerNachfolger.loescheMaximum ();
                
                zeichneNeu (baumposition);

                return this;
            }
        }
        else
        {
            rechterNachfolger = rechterNachfolger.loesche (suchwort, baumposition + "r");
            
            return this;
        }
    }

    /**
     * Nenne den grten Eintrag ab mir im Baum.
     */
    public Eintrag gibMaximum ()
    {
        if (rechterNachfolger instanceof Abschluss)
        {
            return inhalt;
        }
        else
        {
            return rechterNachfolger.gibMaximum ();
        }
    }

    /**
     * Lsche den grten Knoten ab mir im Baum.
     */
    public Baumelement loescheMaximum ()
    {
        if (rechterNachfolger instanceof Abschluss)
        {
            return linkerNachfolger;
        }
        else
        {
            rechterNachfolger = rechterNachfolger.loescheMaximum ();

            return this;
        }
    }

    /**
     * Zeichne den Baum ab mir neu.
     */
    public void zeichneNeu (String baumposition)
    {
        inhalt.macheUnsichtbar ();
        inhalt.zeichneAnBaumposition (baumposition);

        linkerNachfolger.zeichneNeu (baumposition + "l");
        rechterNachfolger.zeichneNeu (baumposition + "r");
    }
}
