/**
 * Piktogramm einer Person als abstrakte Klasse.
 * 
 * @author Bernd Gramlich
 * @version 2012-12-20
 */
public abstract class Person implements Eintrag
{
    // Basiskoordinaten auf der Leinwand.
    protected int positionX;
    protected int positionY;
    
    // Bestandteile als Referenzen auf Graphikobjekte.
    protected Kreissektor haupthaar;
    protected Vieleck langhaar;
    protected Ellipse kopf;
    protected Vieleck kragen;
    protected Vieleck jacket;
    protected Vieleck rock;
    protected Vieleck hand1;
    protected Vieleck hand2;
    protected Vieleck hose;
    protected Vieleck schuh1;
    protected Vieleck schuh2;
    protected Beschriftung namensschild;
    protected String name;
    
    // Ein hinfhrender Balken zur graphischen Darstellung im Baum.
    protected Vieleck verbindungslinie;

    /**
     * Konstruktor.
     * Erzeuge ein Piktogramm der Strichstrke a
     * an der Position (x|y) mit den gewnschten Farben.
     */
    public Person (int a, int x, int y, String personenname,
    String hautfarbe, String haarfarbe, String schuhfarbe,
    String kleiderfarbe1, String kleiderfarbe2, String kleiderfarbe3)
    {
        positionX = x;
        positionY = y;
        
        haupthaar    = new Kreissektor ();
        langhaar     = new Vieleck ();
        kopf         = new Ellipse ();
        kragen       = new Vieleck ();
        jacket       = new Vieleck ();
        rock         = new Vieleck ();
        hand1        = new Vieleck ();
        hand2        = new Vieleck ();
        hose         = new Vieleck ();
        schuh1       = new Vieleck ();
        schuh2       = new Vieleck ();
        namensschild = new Beschriftung ();
        
        verbindungslinie = new Vieleck ();

        haupthaar.setzePosition (x + 12*a, y);
        haupthaar.setzeRadius (8*a);
        haupthaar.setzeAnfangswinkel (0);
        haupthaar.setzeGesamtwinkel (180);
        haupthaar.setzeFarbe (haarfarbe);

        langhaar.setzeFarbe (haarfarbe);

        kopf.setzePosition (x + 14*a, y +   2*a);
        kopf.setzeBreite (12*a);
        kopf.setzeHoehe (16*a);
        kopf.setzeFarbe (hautfarbe);

        hand1.setzeFarbe (hautfarbe);
        hand2.setzeFarbe (hautfarbe);

        jacket.setzeFarbe (kleiderfarbe1);

        rock.setzeFarbe (kleiderfarbe2);

        hose.setzeFarbe (kleiderfarbe3);

        schuh1.setzeFarbe (schuhfarbe);
        schuh2.setzeFarbe (schuhfarbe);

        name = personenname;
        namensschild.setzePosition (x, y + 85*a);
        namensschild.setzeSchriftgroesse (5*a);
        namensschild.setzeFarbe (kleiderfarbe1);
        namensschild.setzeInhalt (zentriere (name));
        
        verbindungslinie.setzeFarbe ("schwarz");
    }

    /**
     * Mache das Piktogramm sichtbar.
     */
    public void macheSichtbar ()
    {
        haupthaar.macheSichtbar ();        
        langhaar.macheSichtbar ();
        jacket.macheSichtbar ();
        hand1.macheSichtbar ();
        hand2.macheSichtbar ();
        kragen.macheSichtbar ();
        hose.macheSichtbar ();
        rock.macheSichtbar ();
        kopf.macheSichtbar ();
        schuh1.macheSichtbar ();
        schuh2.macheSichtbar ();
        namensschild.macheSichtbar ();
        
        verbindungslinie.macheSichtbar ();
    }

    /**
     * Mache das Piktogramm sichtbar.
     */
    public void macheUnsichtbar ()
    {
        haupthaar.macheUnsichtbar ();        
        langhaar.macheUnsichtbar ();
        jacket.macheUnsichtbar ();
        hand1.macheUnsichtbar ();
        hand2.macheUnsichtbar ();
        kragen.macheUnsichtbar ();
        rock.macheUnsichtbar ();
        kopf.macheUnsichtbar ();
        hose.macheUnsichtbar ();
        schuh1.macheUnsichtbar ();
        schuh2.macheUnsichtbar ();
        namensschild.macheUnsichtbar ();
        
        verbindungslinie.macheUnsichtbar ();
    }

    /**
     * Zentriere den Namen in einer Zeichenkette von 28 Zeichen.
     * Liefert nur fr kurze Name einigermaen sinnvolle Ergebnisse.
     * Besser wre, wenn die Klasse Beschriftung die Mglichkeit zur
     * exakt zentrierten Ausgabe htte.
     */
    protected String zentriere (String name)
    {
        String zentrierterName = "";

        for (int i = 0; i < (28 - name.length ()) / 2; i++)
        {
            zentrierterName = zentrierterName + " ";
        }

        zentrierterName = zentrierterName + name;

        for (int i = 0; i < (28 - name.length ()) / 2; i++)
        {
            zentrierterName = zentrierterName + " ";
        }

        return zentrierterName;
    }

    /**
     * ndere den Namen des Piktogramms.
     */
    public void setzeName (String neuerName)
    {
        name = neuerName;
        namensschild.setzeInhalt (zentriere (name));
    }

    /**
     * Nenne den Namen des Piktogramms.
     */
    public String gibName ()
    {
        return name;
    }

    /**
     * Verschiebe die Person um den Vektor (dx|dy).
     */
    public void verschiebe (int dx, int dy)
    {
        positionX += dx;
        positionY += dy;
        
        macheUnsichtbar ();

        haupthaar.verschiebeWaagrecht (dx);
        haupthaar.verschiebeSenkrecht (dy);
        langhaar.verschiebe (dx, dy);
        jacket.verschiebe (dx, dy);
        hand1.verschiebe (dx, dy);
        hand2.verschiebe (dx, dy);
        kragen.verschiebe (dx, dy);
        rock.verschiebe (dx, dy);
        kopf.verschiebeWaagrecht (dx);
        kopf.verschiebeSenkrecht (dy);
        hose.verschiebe (dx, dy);
        schuh1.verschiebe (dx, dy);
        schuh2.verschiebe (dx, dy);
        namensschild.verschiebeWaagrecht (dx);
        namensschild.verschiebeSenkrecht (dy);

        macheSichtbar ();
    }
    
    /**
     * Verschiebe die Person an Position (x|y).
     */
    public void setzePosition (int x, int y)
    {
        verschiebe (x - positionX, y - positionY);
    }

    /**
     * Gib an, ob dieses Piktogramm weiblich ist.
     */
    public abstract boolean istWeiblich ();

    /**
     * Nenne die Haarfarbe.
     */
    public String gibHaarfarbe ()
    {
        return haupthaar.gibFarbe ();
    }

    /**
     * Nenne die erste Kleiderfarbe.
     */
    public String gibKleiderfarbe1 ()
    {
        return jacket.gibFarbe ();
    }

    /**
     * Nenne die zweite Kleiderfarbe.
     */
    public String gibKleiderfarbe2 ()
    {
        return rock.gibFarbe ();
    }

    /**
     * Nenne die dritte Kleiderfarbe.
     */
    public String gibKleiderfarbe3 ()
    {
        return hose.gibFarbe ();
    }

    /**
     * Erzeuge eine zufllige Person der Strichstrke 2 und
     * zeichne sie unsichtbar in der linken oberen Leinwandecke.
     */
    public static Person erzeugeZufaellig ()
    {
        boolean weiblich;
        String name;
        String hautfarbe;
        String haarfarbe;
        String schuhfarbe;
        String kleiderfarbe1;
        String kleiderfarbe2;
        String kleiderfarbe3;

        int zufall;
        
        zufall = (int) Math.floor (2 * Math.random ());
        
        switch (zufall)
        {
            case 0:  weiblich = false; break;
            default: weiblich = true; break;
        }
        
        zufall = (int) Math.floor (10 * Math.random ());

        if (weiblich)
        {
            switch (zufall)
            {
                case 0:  name = "Mia"; break;
                case 1:  name = "Emma"; break;
                case 2:  name = "Hanna"; break;
                case 3:  name = "Anna"; break;
                case 4:  name = "Lea"; break;
                case 5:  name = "Leonie"; break;
                case 6:  name = "Lina"; break;
                case 7:  name = "Marie"; break;
                case 8:  name = "Sophia"; break;
                default: name = "Lena"; break;
            }
        }
        else
        {
            switch (zufall)
            {
                case 0:  name = "Ben"; break;
                case 1:  name = "Leon"; break;
                case 2:  name = "Lucas"; break;
                case 3:  name = "Finn"; break;
                case 4:  name = "Jonas"; break;
                case 5:  name = "Maximilian"; break;
                case 6:  name = "Louis"; break;
                case 7:  name = "Paul"; break;
                case 8:  name = "Felix"; break;
                default: name = "Luca"; break;
            }
        }

        zufall = (int) Math.floor (10 * Math.random ());

        switch (zufall)
        {
            case 0:  hautfarbe = "gebrunt"; break;
            case 1:  hautfarbe = "hellbraun"; break;
            default: hautfarbe = "bleich"; break;
        }

        zufall = (int) Math.floor (10 * Math.random ());

        switch (zufall)
        {
            case 0:  haarfarbe = "rot"; break;
            case 1:
            case 2:
            case 3:  haarfarbe = "blond"; break;
            case 4:
            case 5:
            case 6:  haarfarbe = "braun"; break;
            default: haarfarbe = "schwarz"; break;
        }

        zufall = (int) Math.floor (10 * Math.random ());

        if (weiblich)
        {
            switch (zufall)
            {
                case 0:
                case 1:  kleiderfarbe1 = "rot"; break;
                case 2:
                case 3:  kleiderfarbe1 = "pink"; break;
                case 4:
                case 5:  kleiderfarbe1 = "lila"; break;
                case 6:  kleiderfarbe1 = "gelb"; break;
                case 7:  kleiderfarbe1 = "orange"; break;
                case 8:  kleiderfarbe1 = "trkis"; break;
                default: kleiderfarbe1 = "braun"; break;
            }
        }
        else
        {
            switch (zufall)
            {
                case 0:
                case 1: 
                case 2:  kleiderfarbe1 = "grau"; break;
                case 3: 
                case 4:  kleiderfarbe1 = "braun"; break;
                case 5:
                case 6:  kleiderfarbe1 = "dunkelbraun"; break;
                case 7: 
                case 8:  kleiderfarbe1 = "schwarz"; break;
                default: kleiderfarbe1 = "blau"; break;
            }
        }

        zufall = (int) Math.floor (10 * Math.random ());

        if (weiblich)
        {
            switch (zufall)
            {
                case 0: 
                case 1:  kleiderfarbe3 = "grau"; break;
                case 2:
                case 3:  kleiderfarbe3 = "hellbraun"; break;
                case 4:
                case 5:  kleiderfarbe3 = "schwarz"; break;
                case 6:  kleiderfarbe3 = kleiderfarbe1; break;
                default: kleiderfarbe3 = hautfarbe; break;
            }
        }
        else
        {
            switch (zufall)
            {
                case 0:  kleiderfarbe3 = "grau"; break;
                case 1:  kleiderfarbe3 = "braun"; break;
                case 2:  kleiderfarbe3 = "dunkelbraun"; break;
                case 3:  kleiderfarbe3 = "schwarz"; break;
                case 4:
                case 5:
                case 6:  kleiderfarbe3 = "blau"; break;
                default: kleiderfarbe3 = kleiderfarbe1;
            }
        }

        zufall = (int) Math.floor (10 * Math.random ());

        if (weiblich)
        {
            switch (zufall)
            {
                case 0:  kleiderfarbe2 = "rot"; break;
                case 1:  kleiderfarbe2 = "pink"; break;
                case 2:  kleiderfarbe2 = "lila"; break;
                case 3:  kleiderfarbe2 = "gelb"; break;
                case 4:  kleiderfarbe2 = "trkis"; break;
                default: kleiderfarbe2 = kleiderfarbe1; break;
            }
        }
        else
        {
            switch (zufall)
            {
                case 0:  kleiderfarbe2 = "rot"; break;
                case 1:  kleiderfarbe2 = "pink"; break;
                case 2:  kleiderfarbe2 = "orange"; break;
                case 3:  kleiderfarbe2 = "grn"; break;
                case 4:  kleiderfarbe2 = "trkis"; break;
                case 5:  kleiderfarbe2 = "blau"; break;
                case 6:  kleiderfarbe2 = "lila"; break;
                case 7:  kleiderfarbe2 = "grau"; break;
                case 8:  kleiderfarbe2 = "braun"; break;
                default: kleiderfarbe2 = "schwarz"; break;
            }
        }

        zufall = (int) Math.floor (10 * Math.random ());

        if (weiblich)
        {
            switch (zufall)
            {
                case 0:  schuhfarbe = "schwarz"; break;
                case 1:  
                case 2:  schuhfarbe = "hellbraun"; break;
                case 3:  
                case 4:  schuhfarbe = "dunkelbraun"; break;
                case 5:
                case 6:
                case 7:  schuhfarbe = "braun"; break;
                default: schuhfarbe = kleiderfarbe1; break;
            }
        }
        else
        {
            switch (zufall)
            {
                case 0:
                case 1:  schuhfarbe = "schwarz"; break;
                case 2: 
                case 3:  schuhfarbe = "hellbraun"; break;
                case 4:
                case 5:  schuhfarbe = "dunkelbraun"; break;
                default: schuhfarbe = "braun"; break;
            }
        }

        if (weiblich)
        {
            return new Frau (2, 0, 0, name, hautfarbe, haarfarbe, schuhfarbe, 
                kleiderfarbe1, kleiderfarbe2, kleiderfarbe3);
        }
        else
        {
            return new Mann (2, 0, 0, name, hautfarbe, haarfarbe, schuhfarbe, 
                kleiderfarbe1, kleiderfarbe2, kleiderfarbe3);
        }
    }
        
    /**
     * Gib an, ob diese Person kleiner als ein anderer Eintrag ist.
     * Entscheide also, ob ihr Name alphabetisch vor dem Namen des
     * anderen Eintrags kommt.
     * 
     * @return true, falls die Person alphabetisch vor
     *         "andererEintrag" kommt.
     *         false, falls die Person alphabetisch an derselben
     *         Stelle oder hinter "andererEintrag" kommt oder falls
     *         "andererEintrag" keine Person ist.
     */
    public boolean istKleinerAls (Eintrag andererEintrag)
    {
        if (andererEintrag instanceof Person)
        {
            /*
             * Wenn der andere Eintrag tatschlich eine Person ist ...
             */
            if (name.compareTo (((Person) andererEintrag).gibName ()) < 0)
            {
                /*
                 * ... und mein Name vor dem Namen des anderen Eintrags
                 * kommt, gib true zurck.
                 */
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }
    
    /**
     * Drucke eine Beschreibung der Person auf die Konsole.
     */
    public void druckeAufKonsole ()
    {
        System.out.println ("Name      : " + name);
        /*
         * Weitere Steckbriefdaten sind momentan ausgeblendet.
         * 
        System.out.print   ("Geschlecht: ");
        if (istWeiblich ())
        {
            System.out.print ("weiblich");
        }
        else
        {
            System.out.print ("mnnlich");
        }
        System.out.println ("Hautfarbe : " + hand1.gibFarbe ());
        System.out.println ("Haarfarbe : " + haupthaar.gibFarbe ());
        System.out.println ();
         */
    }
    
    /**
     * Lasse das Piktogramm an "baumposition" erscheinen. Die
     * "baumposition" ist dabei eine Zeichenkette, die den Pfad
     * von der Wurzel eines Binrbaums zu der gewnschten
     * Position beschreibt. Jedes "l" in der Zeichenkette
     * bedeutet Verzweigung nach links, jedes "r" Verzweigung
     * nach rechts. Die Position wird so berechnet, dass ein
     * Piktogramm der Strichstrke 2 gut zentriert im Baum
     * erscheint.
     */
    public void zeichneAnBaumposition (String baumposition)
    {
        /*
         * Filtere alle Zeichen, die kein "l" oder "r" sind, aus
         * der Position heraus.
         */
        String position = "";
        
        for (int i = 0; i < baumposition.length (); i++)
        {
            switch (baumposition.charAt (i))
            {
                case 'l':
                case 'r': position = position + baumposition.charAt (i); break;
            }
        }
        
        /*
         * Lege Baumbreite und Einzelhhe fest.
         */
        final int baumbreite  = 1024;
        final int einzelhoehe = 190;
        
        /*
         * Durchlaufe den Baumpfad von der Wurzel bis zur
         * "position" und speichere dabei in (x|y) stets
         * den Scheitelpunkt der aktuellen Person und in
         * (xVorgaenger|yVorgaenger) den Scheitelpunkt
         * ihres Vorgngers.
         */
        int einzelbreite = baumbreite / 2;
        
        int xVorgaenger  = einzelbreite;
        int yVorgaenger  = 0;
        int x = xVorgaenger;
        int y = yVorgaenger;
        
        for (int i = 0; i < position.length (); i++)
        {
            einzelbreite = einzelbreite / 2;
            xVorgaenger  = x;
            yVorgaenger  = y;
            
            switch (position.charAt (i))
            {
                case 'l': x = xVorgaenger - einzelbreite; break; 
                case 'r': x = xVorgaenger + einzelbreite; break;
            }
            
            y = yVorgaenger + einzelhoehe;
        }
        
        /*
         * Zeichne eine Verbindungsline zwischen dem Standpunkt
         * meines Vorgngers und meinem Scheitelpunkt.
         * 
         * Wenn ich in der Wurzel steht, entartet die Linie zu
         * einem Rechteck der Hhe 0, wird also unsichtbar.
         */
        verbindungslinie.loescheAlleEcken ();
        verbindungslinie.ergaenzeEcke (xVorgaenger - 5, y - 15);
        verbindungslinie.ergaenzeEcke (x - 5, y);
        verbindungslinie.ergaenzeEcke (x + 5, y);
        verbindungslinie.ergaenzeEcke (xVorgaenger + 5, y - 15);
        verbindungslinie.macheSichtbar ();
        
        /*
         * Zeichne mich.
         */
        setzePosition (x - 40, y);
        macheSichtbar ();
    }
    
    /**
     * Gib an, ob dieses Piktogramm mnnlich ist.
     */
    public boolean istMaennlich ()
    {
        return !istWeiblich ();
    }
    
    /**
     * Gib an, ob dieses Piktogramm eine Blondine darstellt.
     */
    public boolean istBlondine ()
    {
        return istWeiblich () && "blond" == gibHaarfarbe ();
    }
      
    /**
     * Gib an, ob dieses Piktogramm einen Anzugtrger darstellt.
     */
    public boolean istAnzugtraeger ()
    {
        return istMaennlich () &&
               jacket.gibFarbe ().equals (hose.gibFarbe ());
    }
    
    /**
     * Besttige, dass das Piktogramm beliebig ist.
     */
    public boolean istBeliebig ()
    {
        return true;
    }
    
    /**
     * Gib die Lnge des Namens.
     */
    public int gibNamenslaenge ()
    {
        return name.length ();
    }
    
    /**
     * Frbe die Haare in der gewnschten Farbe.
     */
    public void faerbeHaare (String neueFarbe)
    {
        haupthaar.setzeFarbe (neueFarbe);
        langhaar.setzeFarbe (neueFarbe);
        kopf.macheSichtbar ();
    }
    
    /**
     * Frbe das Jacket in der gewnschten Farbe.
     */
    public void setzeJacketfarbe (String neueFarbe)
    {
        jacket.setzeFarbe (neueFarbe);
        rock.macheSichtbar ();
    }
    
    /**
     * Schwebe um 10 Bildpunkte nach oben.
     */
    public void schwebe ()
    {
        verschiebe (0, -10);
    }
    
    /**
     * Sinke um 10 Bildpunkte nach unten.
     */
    public void sinke ()
    {
        verschiebe (0, 10);
    }
    
    /**
     * Setze die Kleiderfarben.
     */
    public void setzeKleiderfarben (String neueFarbe1, String neueFarbe2, String neueFarbe3)
    {
        jacket.setzeFarbe (neueFarbe1);
        hose.setzeFarbe (neueFarbe3);
        rock.setzeFarbe (neueFarbe2);
    }
    
    /**
     * Setze die Schuhfarbe.
     */
    public void setzeSchuhfarbe (String neueFarbe)
    {
        schuh1.setzeFarbe (neueFarbe);
        schuh2.setzeFarbe (neueFarbe);
    }
    
    /**
     * Schreibe den Namen gro.
     */
    public void schreibeNameGross ()
    {
        name = name.toUpperCase ();
    }
    
    
    /**
     * Vergleiche den Namen der Person mit dem Suchwort.
     * 
     * @param suchwort Der gesuchte Begriff.
     * @return eine negative Zahl, wenn der Name alphabetisch vor dem Suchwort kommt.
     *         0, wenn der Name mit dem Suchwort bereinstimmt.
     *         eine positive Zahl, wenn der Name alphabetisch nach dem Suchwort kommt.
     */
    public int vergleicheMit (String suchwort)
    {
        return name.compareTo (suchwort);
    }
}